/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#ifndef BACKEND_H
#define BACKEND_H

#include "QtCore/qcontainerfwd.h"
#include "QtCore/qtmetamacros.h"
#include "QtCore/qvariant.h"
#include "core/Mirai.h"
#include "core/TasksView.h"
#include <QtCore/QDateTime>
#include <QtCore/QDir>
#include <QtCore/QJsonArray>
#include <QtCore/QJsonDocument>
#include <QtCore/QJsonObject>
#include <QtCore/QList>
#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtQml/qqmlregistration.h>
#include <memory>
#include <qmap.h>

#include "TaskItem.h"

class Backend : public QObject
{
	Q_OBJECT
	QML_ELEMENT
	Q_PROPERTY(QVariant tasks READ getTasks NOTIFY tasksChanged)
	Q_PROPERTY(QVariant files READ getFiles NOTIFY filesChanged)
	Q_PROPERTY(QVariant tags READ getTags NOTIFY tasksChanged)
	Q_PROPERTY(QVariant activeTagsFilter READ getActiveTagsFilter NOTIFY tasksChanged)
	Q_PROPERTY(QVariant activeFilesFilter READ getActiveFilesFilter NOTIFY tasksChanged)
	Q_PROPERTY(bool shouldHideCompletedTasks READ shouldHideCompletedTasks WRITE hideCompletedTasks
				   NOTIFY tasksChanged)

  public:
	Backend();

	Q_INVOKABLE void addTodo(QString newTodo, QString date);
	Q_INVOKABLE void addTodoFromRawFormat(QString filePath, QString text, QString date);
	Q_INVOKABLE void addTagFilter(QString tag);
	Q_INVOKABLE void removeTagFilter(QString tag);
	Q_INVOKABLE void addFileFilter(QString fileName);
	Q_INVOKABLE void removeFileFilter(QString fileName);
	Q_INVOKABLE void removeFilters();
	Q_INVOKABLE void updateTodoFromRawFormat(int todoIndex, QString text, QString date);
	Q_INVOKABLE void updateTodo(int todoIndex, QString state, QString text, QString date);
	Q_INVOKABLE void removeTodo(int todoIndex);
	Q_INVOKABLE void hideCompletedTasks(bool shouldHide);
	Q_INVOKABLE QString getTagColor(QString tag);

  private:
	void rebuildQMLTasksList();

	QVariant getTasks();
	QVariant getFiles();
	QVariant getTags();
	QVariant getActiveTagsFilter();
	QVariant getActiveFilesFilter();
	bool shouldHideCompletedTasks();

	mirai::Mirai mirai;
	std::weak_ptr<mirai::TasksView> view;

	QList<QMLTaskItem> QMLTasks;
	QList<QMLTasksFile> QMLTasksFiles;
	QList<QString> QMLTags;
	QList<QString> QMLActiveTagsFilter;
	QList<QString> QMLActiveFilesFilter;
	QMap<QString, QString> tagsConfig;

	bool shouldHideCompletedTasks_ = true;

  signals:

	void tasksChanged();
	void filesChanged();
};

#endif
