/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#ifndef MIRAI_TASKSFILE_H
#define MIRAI_TASKSFILE_H

#include "TaskItem.h"
#include <memory>
#include <string>
#include <vector>

namespace mirai
{

using TasksPtrs = std::vector<std::unique_ptr<TaskItem>>;

struct TasksFileConfig {
	std::string name;
	std::string path;
};

struct TasksFileConstructor {
	const std::string &name;
	const std::string &path;
	std::vector<TaskItem> tasks;
};

class TasksFile
{

  public:
	TasksFile(const TasksFile &) = delete;
	TasksFile &operator=(const TasksFile &) = delete;
	TasksFile(TasksFileConstructor params);
	void addTask(std::string text, std::string date);
	void addTask(TaskItemData TaskItem);
	void removeTask(const TaskItem *taskToRemove);
	void sortByDate();

	const std::string &getName() const;
	const std::string &getPath() const;
	TasksPtrs &getTasks();

	void setDirty(bool shouldBeDirty);
	bool isDirty() const;

  private:
	void onTaskChanged(TaskItem &);

	std::string name;
	std::string path;
	TasksPtrs tasks;
	bool isDirty_ = false;
};
} // namespace mirai

#endif
